package com.abstractsense.paddlefriendly.data;

/**
 * Connexió a la base de dades.
 * Josep Sanromà
 * 11/12/2016
 */

import com.abstractsense.paddlefriendly.business.DadesTabulades;
import com.abstractsense.paddlefriendly.business.Game;
import com.abstractsense.paddlefriendly.business.GameScore;
import com.abstractsense.paddlefriendly.business.Payment;
import com.abstractsense.paddlefriendly.business.Player;
import static com.abstractsense.paddlefriendly.data.Constants.*;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.UnknownHostException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

public class Database {

    public Database() {

    }

    public Player login(String email, String pass) {
        String strIn = urlConnect(URL_LOGIN + URL_PARAM_USUA + email + URL_PARAM_PASS + pass);
        return jsonToPlayer(strIn);
    }

    /*
    public Game gameDetails(GlobalVars globals, int gameId) {
        Player player = globals.getPlayer();
        String strIn = urlConnect(URL_GAME + URL_PARAM_USUA + player.getUsua() + URL_PARAM_PASS + player.getPass() + URL_PARAM_GAME + gameId);
        return jsonToGame(strIn);
    }
    */

    public List<Player> playerList(GlobalVars globals, int grupId) {
        Player player = globals.getPlayer();
        String strIn = urlConnect(URL_PLAYERS + URL_PARAM_USUA + player.getUsua() + URL_PARAM_PASS + player.getPass() + URL_PARAM_GRUP + grupId);
        List<Player> players = new ArrayList<Player>();

        try {
            JSONArray jsonArray = new JSONArray(strIn);
            for(int i=0; i<jsonArray.length(); i++){
                JSONObject obj = jsonArray.getJSONObject(i);
                strIn = obj.toString();
                players.add(jsonToPlayer(strIn));
            }
        } catch(JSONException e) {
            e.printStackTrace();
        }
        return players;
    }

    public List<Game> gameList(GlobalVars globals, int grupId) {
        Player player = globals.getPlayer();
        String strIn = urlConnect(URL_GAMES + URL_PARAM_USUA + player.getUsua() + URL_PARAM_PASS + player.getPass() + URL_PARAM_GRUP + grupId);
        List<Game> games = new ArrayList<Game>();

        try {
            JSONArray jsonArray = new JSONArray(strIn);
            for(int i=0; i<jsonArray.length(); i++){
                JSONObject obj = jsonArray.getJSONObject(i);
                strIn = obj.toString();
                games.add(jsonToGame(strIn));
            }
        } catch(JSONException e) {
            e.printStackTrace();
        }
        return games;
    }

    public int updatePlayerGame(GlobalVars globals, int gameId, String operation) {
        Player player = globals.getPlayer();
        String strIn = urlConnect(URL_PLAYER_GAME + URL_PARAM_USUA + player.getUsua() + URL_PARAM_PASS + player.getPass() + URL_PARAM_GAME + gameId + URL_PARAM_OPERATION + operation) ;
        return jsonToInt(strIn);
    }

    public boolean saveGame(GlobalVars globals, Game game) {
        Player player = globals.getPlayer();
        DateFormat df = new SimpleDateFormat(DATE_FORMAT_MYSQL);
        String strIn = urlConnect(URL_SAVE_GAME + URL_PARAM_USUA + player.getUsua() + URL_PARAM_PASS + player.getPass()
                        + URL_PARAM_GAME + game.getIdPartit()
                        + URL_PARAM_DATA + df.format(game.getData())
                        + URL_PARAM_TIME + game.getHora()
                        + URL_PARAM_LLOC + game.getLloc()
                        + URL_PARAM_GRUP + game.getGrup()
                        + URL_PARAM_COST + game.getCost()
                        + URL_PARAM_SET1A + game.getPuntuacio().getSet1A()
                        + URL_PARAM_SET1B + game.getPuntuacio().getSet1B()
                        + URL_PARAM_SET2A + game.getPuntuacio().getSet2A()
                        + URL_PARAM_SET2B + game.getPuntuacio().getSet2B()
                        + URL_PARAM_SET3A + game.getPuntuacio().getSet3A()
                        + URL_PARAM_SET3B + game.getPuntuacio().getSet3B()
                        );
        return jsonToBoolean(strIn);
    }

    public List<Payment> payments(GlobalVars globals, Game game) {
        Player player = globals.getPlayer();
        String strIn = urlConnect(URL_PAGAMENTS + URL_PARAM_USUA + player.getUsua() + URL_PARAM_PASS + player.getPass() + URL_PARAM_GAME + game.getIdPartit());
        List<Payment> payments = new ArrayList<Payment>();

        try {
            JSONArray jsonArray = new JSONArray(strIn);
            for(int i=0; i<jsonArray.length(); i++){
                JSONObject obj = jsonArray.getJSONObject(i);
                strIn = obj.toString();
                payments.add(jsonToPayment(strIn));
            }
        } catch(JSONException e) {
            e.printStackTrace();
        }
        return payments;
    }

    public boolean addPayment(GlobalVars globals, Payment payment) {
        Player player = globals.getPlayer();
        DateFormat df = new SimpleDateFormat(DATE_FORMAT_MYSQL);
        String strIn = urlConnect(URL_ADD_PAYMENT + URL_PARAM_USUA + player.getUsua() + URL_PARAM_PASS + player.getPass()
                + URL_PARAM_GAME + payment.getIdPartit()
                + URL_PARAM_DATA + df.format(payment.getData())
                + URL_PARAM_PLAYER + payment.getPlayer()
                + URL_PARAM_IMPORT + payment.getQuantitat()
                );
        return jsonToBoolean(strIn);
    }

    public List<DadesTabulades> saldos(GlobalVars globals, int grup) {
        Player player = globals.getPlayer();
        String strIn = urlConnect(URL_SALDOS + URL_PARAM_USUA + player.getUsua() + URL_PARAM_PASS + player.getPass() + URL_PARAM_GRUP + grup);
        return jsonToDT(strIn);
    }

    public List<DadesTabulades> estadistiques(GlobalVars globals, int grup) {
        Player player = globals.getPlayer();
        String strIn = urlConnect(URL_ESTADISTIQUES + URL_PARAM_USUA + player.getUsua() + URL_PARAM_PASS + player.getPass()  + URL_PARAM_GRUP + grup);
        List<DadesTabulades> dt = jsonToDT(strIn);
        return dt;
    }

    /**
     *  JSON to * methods.
     *
     */
    private Player jsonToPlayer(String jsonString) {
        Player player = null;
        try {
            JSONObject obj = new JSONObject(jsonString);
            player = new Player(obj.getString("usua"), obj.getString("pass"), obj.getString("noms"), obj.getString("cogs"), obj.getString("sexe"), obj.getString("perm"), Integer.parseInt(obj.getString("nivell")));
        } catch(JSONException e) {
            e.printStackTrace();
        }
        return player;
    }

    private Game jsonToGame(String jsonString) {
        Game game = null;
        try {
            JSONObject obj = new JSONObject(jsonString);
            game = new Game(Integer.parseInt(obj.getString("idpartit")),obj.getString("data"), obj.getString("time"), obj.getString("lloc"), GRUP, obj.getString("propietari"), Float.parseFloat(obj.getString("cost")));
            try {
                DateFormat df = new SimpleDateFormat(DATE_FORMAT_MYSQL);
                game.setData(df.parse(obj.getString("data")));
            } catch (ParseException e) {
                e.printStackTrace();
            }
            game.setJugadorA1(obj.getString("jugadorA1"));
            game.setJugadorA2(obj.getString("jugadorA2"));
            game.setJugadorB1(obj.getString("jugadorB1"));
            game.setJugadorB2(obj.getString("jugadorB2"));
            game.setRebutjats(Integer.parseInt(obj.getString("rebutjats")));
            //game.setJoRebutjo(Boolean.parseBoolean(obj.getString("jorebutjo")!=""));
            if(!obj.getString("jorebutjo").equals("")) {
                game.setJoRebutjo(true);
            } else {
                game.setJoRebutjo(false);
            }
            GameScore gameScore = new GameScore(obj.getInt("set1A"),obj.getInt("set1B"),
                                                obj.getInt("set2A"),obj.getInt("set2B"),
                                                obj.getInt("set3A"),obj.getInt("set3B"));
            game.setPuntuacio(gameScore);

        } catch(JSONException e) {
            e.printStackTrace();
        }
        return game;
    }

    private Payment jsonToPayment(String jsonString) {
        Payment payment = null;
        try {
            JSONObject obj = new JSONObject(jsonString);
            payment = new Payment(Integer.parseInt(obj.getString("idpagament")), Integer.parseInt(obj.getString("idpartit")),obj.getString("data"), obj.getString("jugador"), Float.parseFloat(obj.getString("import")), obj.getString("nomjugador"));
            try {
                DateFormat df = new SimpleDateFormat(DATE_FORMAT_MYSQL);
                payment.setData(df.parse(obj.getString("data")));
            } catch (ParseException e) {
                e.printStackTrace();
            }

        } catch(JSONException e) {
            e.printStackTrace();
        }
        return payment;
    }

    private boolean jsonToBoolean(String jsonString) {
        boolean ret = false;
        try {
            JSONArray arr = new JSONArray(jsonString);
            ret = arr.getBoolean(1);
        } catch(JSONException e) {
            e.printStackTrace();
        }
        return ret;
    }

    private int jsonToInt(String jsonString) {
        int ret = 0;
        try {
            JSONArray arr = new JSONArray(jsonString);
            ret = arr.getInt(1);
        } catch(JSONException e) {
            e.printStackTrace();
        }
        return ret;
    }

    private List<DadesTabulades> jsonToDT(String jsonString) {
        JSONArray jsonArray = null;
        try {
            jsonArray = new JSONArray(jsonString);
        } catch(JSONException e) {
            e.printStackTrace();
        }

        List<DadesTabulades> dts = new ArrayList<DadesTabulades>();
        try {
            for(int i=0; i<jsonArray.length(); i++){
                JSONObject obj = jsonArray.getJSONObject(i);
                String strIn = obj.toString();
                DadesTabulades dt = new DadesTabulades(obj.getString("concepte"), Float.parseFloat(obj.getString("quantitat")), Float.parseFloat(obj.getString("percentatge")));
                dts.add(dt);
            }
        } catch(JSONException e) {
            e.printStackTrace();
        }
        return dts;
    }

    /* Actual connection */
    private String urlConnect(String urlToConnect) {

        InputStream in = null;
        int resCode = -1;

        try {
            URL url = new URL(urlToConnect);
            URLConnection urlConn = url.openConnection();

            if (!(urlConn instanceof HttpURLConnection)) {
                throw new IOException("URL is not an Http URL");
            }

            HttpURLConnection httpConn = (HttpURLConnection) urlConn;
            httpConn.setAllowUserInteraction(false);
            httpConn.setInstanceFollowRedirects(true);
            httpConn.setRequestMethod("GET");
            httpConn.connect();
            resCode = httpConn.getResponseCode();

            if (resCode == HttpURLConnection.HTTP_OK) {
                in = httpConn.getInputStream();
            }
        }catch (UnknownHostException e) {
            e.printStackTrace();
        }catch (MalformedURLException e) {
            e.printStackTrace();
        }catch (IOException e) {
            e.printStackTrace();
        }

        java.util.Scanner scanner = new java.util.Scanner(in,"UTF-8").useDelimiter("\n");
        String strIn = scanner.hasNext() ? scanner.next() : "";

        return strIn;
    }

}
