package com.abstractsense.paddlefriendly.presentation;

import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.app.ActionBar;
import android.support.v4.app.NavUtils;
import android.view.MenuItem;
import android.widget.TextView;

import com.abstractsense.paddlefriendly.R;
import com.abstractsense.paddlefriendly.business.Game;
import com.abstractsense.paddlefriendly.business.GameScore;
import com.abstractsense.paddlefriendly.data.Database;
import com.abstractsense.paddlefriendly.data.GlobalVars;

import static com.abstractsense.paddlefriendly.data.Constants.GRUP;

/**
 * An activity representing a single Game detail screen. This
 * activity is only used narrow width devices. On tablet-size devices,
 * item details are presented side-by-side with a list of items
 * in a {@link GameListActivity}.
 */
public class GameDetailActivity extends AppCompatActivity {

    private GameDetailActivity.SaveGameTask mSaveGameTask = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_game_detail);
        Toolbar toolbar = (Toolbar) findViewById(R.id.detail_toolbar);
        setSupportActionBar(toolbar);

        FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fab);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                startActivity(new Intent(GameDetailActivity.this, PaymentsActivity.class));
            }
        });

        // Show the Up button in the action bar.
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setDisplayHomeAsUpEnabled(true);
        }

        // savedInstanceState is non-null when there is fragment state
        // saved from previous configurations of this activity
        // (e.g. when rotating the screen from portrait to landscape).
        // In this case, the fragment will automatically be re-added
        // to its container so we don't need to manually add it.
        // For more information, see the Fragments API guide at:
        //
        // http://developer.android.com/guide/components/fragments.html
        //
        if (savedInstanceState == null) {
            // Create the detail fragment and add it to the activity
            // using a fragment transaction.
            Bundle arguments = new Bundle();
            arguments.putString(GameDetailFragment.ARG_ITEM_ID,
                    getIntent().getStringExtra(GameDetailFragment.ARG_ITEM_ID));
            GameDetailFragment fragment = new GameDetailFragment();
            fragment.setArguments(arguments);
            getSupportFragmentManager().beginTransaction()
                    .add(R.id.game_detail_container, fragment)
                    .commit();
        }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        if (id == android.R.id.home) {
            // This ID represents the Home or Up button. In the case of this
            // activity, the Up button is shown. Use NavUtils to allow users
            // to navigate up one level in the application structure. For
            // more details, see the Navigation pattern on Android Design:
            //
            // http://developer.android.com/design/patterns/navigation.html#up-vs-back

            //NavUtils.navigateUpTo(this, new Intent(this, GameListActivity.class));
            backToList();

            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onBackPressed() {
        backToList();
    }

    private void backToList() {

        // Get idPartit.
        GlobalVars globals = ((GlobalVars)getApplication());
        int idPartit = globals.getGame().getIdPartit();

        // Get data from views.
        String data = ((TextView) findViewById(R.id.data)).getText().toString();
        String hora = ((TextView) findViewById(R.id.hora)).getText().toString();
        String lloc = ((TextView) findViewById(R.id.lloc)).getText().toString();
        float cost = Float.parseFloat(((TextView) findViewById(R.id.cost)).getText().toString());
        int set1A = Integer.decode(((TextView) findViewById(R.id.set1A)).getText().toString());
        int set1B = Integer.decode(((TextView) findViewById(R.id.set1B)).getText().toString());
        int set2A = Integer.decode(((TextView) findViewById(R.id.set2A)).getText().toString());
        int set2B = Integer.decode(((TextView) findViewById(R.id.set2B)).getText().toString());
        int set3A = Integer.decode(((TextView) findViewById(R.id.set3A)).getText().toString());
        int set3B = Integer.decode(((TextView) findViewById(R.id.set3B)).getText().toString());

        // Set game data.
        Game game = new Game(idPartit, data, hora, lloc, GRUP, globals.getPlayer().getUsua(), cost);
        GameScore gameScore = new GameScore(set1A, set1B, set2A, set2B, set3A, set3B);
        game.setPuntuacio(gameScore);

        // Start savedate task.
        mSaveGameTask = new GameDetailActivity.SaveGameTask();
        mSaveGameTask.execute(game);

    }

    /**
     * Connexió a la base de dades per actualitzar camps "apuntar-se" i "no puc".
     */
    public class SaveGameTask extends AsyncTask<Game, Void, Boolean> {

        SaveGameTask() {
        }

        @Override
        protected Boolean doInBackground(Game... game) {

            // Save game.
            GlobalVars globals = ((GlobalVars)getApplication());
            Database db = new Database();
            boolean ret = db.saveGame(globals, game[0]);
            return (ret);
        }

        @Override
        protected void onPostExecute(final Boolean result) {
            mSaveGameTask = null;

            // Go back to games list.
            if(result) {
                NavUtils.navigateUpTo(GameDetailActivity.this, new Intent(GameDetailActivity.this, GameListActivity.class));
            } else {
                // Error message not necessary.
            }
        }

        @Override
        protected void onCancelled() {
            mSaveGameTask = null;
        }
    }
}
