package com.abstractsense.paddlefriendly.presentation;

import android.app.Activity;
import android.app.Dialog;
import android.app.DialogFragment;
import android.app.TimePickerDialog;
import android.os.Bundle;
import android.support.design.widget.CollapsingToolbarLayout;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.TimePicker;

import com.abstractsense.paddlefriendly.R;
import com.abstractsense.paddlefriendly.business.Game;
import com.abstractsense.paddlefriendly.business.GameScore;
import com.abstractsense.paddlefriendly.data.GlobalVars;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import static com.abstractsense.paddlefriendly.data.Constants.DATE_FORMAT;
import static com.abstractsense.paddlefriendly.data.Constants.FLOAT_FORMAT;

/**
 * A fragment representing a single Game detail screen.
 * This fragment is either contained in a {@link GameListActivity}
 * in two-pane mode (on tablets) or a {@link GameDetailActivity}
 * on handsets.
 */
public class GameDetailFragment extends Fragment {
    /**
     * The fragment argument representing the item ID that this fragment
     * represents.
     */
    public static final String ARG_ITEM_ID = "item_id";

    /**
     * Game being edited.
     */
    private Game game = null;

    /**
     * Mandatory empty constructor for the fragment manager to instantiate the
     * fragment (e.g. upon screen orientation changes).
     */
    public GameDetailFragment() {
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        //if (getArguments().containsKey(ARG_ITEM_ID)) {

            GlobalVars globals = ((GlobalVars)getActivity().getApplication());
            game = globals.getGame();

            Activity activity = this.getActivity();
            CollapsingToolbarLayout appBarLayout = (CollapsingToolbarLayout) activity.findViewById(R.id.toolbar_layout);
            if (appBarLayout != null) {
                appBarLayout.setTitle(game.toString());
            }

        //}
    }

    public static class TimePickerFragment extends DialogFragment
            implements TimePickerDialog.OnTimeSetListener {

        @Override
        public Dialog onCreateDialog(Bundle savedInstanceState) {
            // Use the current time as the default values for the picker
            final Calendar c = Calendar.getInstance();
            int hour = c.get(Calendar.HOUR_OF_DAY);
            int minute = c.get(Calendar.MINUTE);

            // Create a new instance of TimePickerDialog and return it
            return new TimePickerDialog(getActivity(), this, hour, minute, true);
        }

        public void onTimeSet(TimePicker view, int hourOfDay, int minute) {
            // Do something with the time chosen by the user
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.game_detail, container, false);

        DateFormat df = new SimpleDateFormat(DATE_FORMAT);

        if (game != null) {
            if(game.getData() != null) {
                ((TextView) rootView.findViewById(R.id.data)).setText(df.format(game.getData()));
            }
            ((TextView) rootView.findViewById(R.id.hora)).setText(game.getHora());
            ((TextView) rootView.findViewById(R.id.lloc)).setText(game.getLloc());
            ((TextView) rootView.findViewById(R.id.cost)).setText(String.format(FLOAT_FORMAT, game.getCost()));
            ((TextView) rootView.findViewById(R.id.jugadorA1)).setText(String.valueOf(game.getJugadorA1()));
            ((TextView) rootView.findViewById(R.id.jugadorA2)).setText(String.valueOf(game.getJugadorA2()));
            ((TextView) rootView.findViewById(R.id.jugadorB1)).setText(String.valueOf(game.getJugadorB1()));
            ((TextView) rootView.findViewById(R.id.jugadorB2)).setText(String.valueOf(game.getJugadorB2()));
            ((TextView) rootView.findViewById(R.id.rebutgen)).setText(String.valueOf(game.getRespostes()));
            GameScore score = game.getPuntuacio();
            ((TextView) rootView.findViewById(R.id.set1A)).setText(String.valueOf(score.getSet1A()));
            ((TextView) rootView.findViewById(R.id.set1B)).setText(String.valueOf(score.getSet1B()));
            ((TextView) rootView.findViewById(R.id.set2A)).setText(String.valueOf(score.getSet2A()));
            ((TextView) rootView.findViewById(R.id.set2B)).setText(String.valueOf(score.getSet2B()));
            ((TextView) rootView.findViewById(R.id.set3A)).setText(String.valueOf(score.getSet3A()));
            ((TextView) rootView.findViewById(R.id.set3B)).setText(String.valueOf(score.getSet3B()));

            // Només el propietari pot modificar.
            GlobalVars globals = (GlobalVars)getActivity().getApplication();
            if(!globals.getGame().getPropietari().equals(globals.getPlayer().getUsua())) {
                ((TextView) rootView.findViewById(R.id.data)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.hora)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.lloc)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.cost)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.jugadorA1)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.jugadorA2)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.jugadorB1)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.jugadorB2)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.rebutgen)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.set1A)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.set1B)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.set2A)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.set2B)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.set3A)).setEnabled(false);
                ((TextView) rootView.findViewById(R.id.set3B)).setEnabled(false);
            }
        }
        return rootView;
    }
}
