package com.abstractsense.paddlefriendly.presentation;

import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.support.design.widget.FloatingActionButton;
//import android.support.design.widget.Snackbar;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

import com.abstractsense.paddlefriendly.R;
import com.abstractsense.paddlefriendly.business.Game;
import static com.abstractsense.paddlefriendly.data.Constants.*;
import com.abstractsense.paddlefriendly.data.Database;
import com.abstractsense.paddlefriendly.data.GlobalVars;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

/**
 * An activity representing a list of Games. This activity
 * has different presentations for handset and tablet-size devices. On
 * handsets, the activity presents a list of items, which when touched,
 * lead to a {@link GameDetailActivity} representing
 * item details. On tablets, the activity presents the list of items and
 * item details side-by-side using two vertical panes.
 */
public class GameListActivity extends AppCompatActivity {

    /**
     * Whether or not the activity is in two-pane mode, i.e. running on a tablet
     * device.
     */
    private boolean mTwoPane;

    /**
     * Tasca asíncrona per enviar i rebre dades.
     */
    private GameListActivity.GetDataTask mGetDataTask = null;
    private GameListActivity.UpdateDataTask mUpdateDataTask = null;
    private GameListActivity.NoPucTask mNoPucTask = null;

    /**
     *  Botó de la llista que premem.
     */
    private View boto = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_game_list);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        toolbar.setTitle(getTitle());

        FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fab);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                /*
                GlobalVars globals = ((GlobalVars)getApplication());
                Player player = globals.getPlayer();
                Snackbar.make(view, "Jugador: "+player.getNoms()+" "+player.getCogs(), Snackbar.LENGTH_LONG)
                        .setAction("Action", null).show();
                */

                // Save in globlas a blank match.
                GlobalVars globals = ((GlobalVars)getApplication());
                Game game = new Game(0, "", "", "", GRUP, globals.getPlayer().getUsua(), 0);
                globals.setGame(game);

                // Call fragment.
                if (mTwoPane) {
                    /*
                    GameDetailFragment fragment = new GameDetailFragment();
                    getSupportFragmentManager().beginTransaction()
                            .replace(R.id.game_detail_container, fragment)
                            .commit();
                            */
                } else {
                    Context context = view.getContext();
                    Intent intent = new Intent(context, GameDetailActivity.class);
                    context.startActivity(intent);
                }
            }
        });

        mGetDataTask = new GetDataTask();
        mGetDataTask.execute((Void) null);

        if (findViewById(R.id.game_detail_container) != null) {
            // The detail container view will be present only in the
            // large-screen layouts (res/values-w900dp).
            // If this view is present, then the
            // activity should be in two-pane mode.
            mTwoPane = true;
        }
    }

    private void setupRecyclerView(@NonNull RecyclerView recyclerView, List<Game> games) {
        recyclerView.setAdapter(new SimpleItemRecyclerViewAdapter(games));
    }

    // Add menu.
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_game_list, menu);
        return true;
    }

    // Menu listener.
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.statistics:
                startActivity(new Intent(GameListActivity.this, StatisticsActivity.class));
                return true;
            case R.id.saldos:
                startActivity(new Intent(GameListActivity.this, SaldosActivity.class));
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    public class SimpleItemRecyclerViewAdapter
            extends RecyclerView.Adapter<SimpleItemRecyclerViewAdapter.ViewHolder> {

        private List<Game> mGames;

        public SimpleItemRecyclerViewAdapter(List<Game> games) {
            mGames = games;
        }


        @Override
        public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = LayoutInflater.from(parent.getContext())
                    .inflate(R.layout.game_list_content, parent, false);
            return new ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final ViewHolder holder, int position) {

            GlobalVars globals = (GlobalVars)getApplication();
            DateFormat df = new SimpleDateFormat(DATE_FORMAT);

            holder.mGameItem = mGames.get(position);
            //holder.mIdView.setText(String.valueOf(mGames.get(position).getIdPartit()));
            //holder.mDataView.setText(mGames.get(position).getData().toString());
            holder.mDataView.setText(df.format(mGames.get(position).getData()));
            holder.mHoraView.setText(mGames.get(position).getHora());
            holder.mLlocView.setText(mGames.get(position).getLloc());
            holder.mRespostesView.setText(String.valueOf(mGames.get(position).getRespostes()));
            holder.mApuntatsView.setText(String.valueOf(mGames.get(position).getApuntats()));

            if(mGames.get(position).getData().before(Calendar.getInstance().getTime())) {
                holder.mApuntarseButton.setEnabled(false);
                holder.mNoPucButton.setEnabled(false);
            }
            if(mGames.get(position).esticApuntat(globals.getPlayer().getUsua())) {
                holder.mApuntarseButton.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.colorApuntarseOn));
            } else {
                holder.mApuntarseButton.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.colorApuntarseOff));
            }
            if(mGames.get(position).joRebutjo()) {
                holder.mNoPucButton.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.colorNoPucOn));
            } else {
                holder.mNoPucButton.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.colorNoPucOff));
            }

            holder.mView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {

                    // Save in globlas the selected match.
                    Game selectedGame = null;
                    Iterator<Game> i;
                    i = mGames.iterator();
                    for(; i.hasNext(); ) {
                        Game game = i.next();
                        if(game.getIdPartit() == holder.mGameItem.getIdPartit()) {
                            selectedGame = game;
                        }
                    }
                    GlobalVars globals = ((GlobalVars)getApplication());
                    globals.setGame(selectedGame);

                    // Call fragment.
                    if (mTwoPane) {
                        //Bundle arguments = new Bundle();
                        //arguments.putString(GameDetailFragment.ARG_ITEM_ID, String.valueOf(holder.mGameItem.getIdPartit()));
                        GameDetailFragment fragment = new GameDetailFragment();
                        //fragment.setArguments(arguments);
                        getSupportFragmentManager().beginTransaction()
                                .replace(R.id.game_detail_container, fragment)
                                .commit();
                    } else {
                        Context context = v.getContext();
                        Intent intent = new Intent(context, GameDetailActivity.class);
                        //intent.putExtra(GameDetailFragment.ARG_ITEM_ID, String.valueOf(holder.mGameItem.getIdPartit()));
                        context.startActivity(intent);
                    }
                }
            });

            holder.mApuntarseButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    boto = v;
                    int idPartit = holder.mGameItem.getIdPartit();
                    mUpdateDataTask = new UpdateDataTask();
                    mUpdateDataTask.execute(new Integer(idPartit));
                }
            });

            holder.mNoPucButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    boto = v;
                    int idPartit = holder.mGameItem.getIdPartit();
                    mNoPucTask = new NoPucTask();
                    mNoPucTask.execute(new Integer(idPartit));

                }
            });
        }

        @Override
        public int getItemCount() {
            return mGames.size();
        }

        public class ViewHolder extends RecyclerView.ViewHolder {
            public final View mView;
            //public final TextView mIdView;
            public final TextView mDataView;
            public final TextView mHoraView;
            public final TextView mLlocView;
            public final TextView mRespostesView;
            public final TextView mApuntatsView;
            public final Button mApuntarseButton;
            public final Button mNoPucButton;
            public Game mGameItem;

            public ViewHolder(View view) {
                super(view);
                mView = view;
                //mIdView = (TextView) view.findViewById(R.id.id);
                mDataView = (TextView) view.findViewById(R.id.data);
                mHoraView = (TextView) view.findViewById(R.id.hora);
                mLlocView = (TextView) view.findViewById(R.id.lloc);
                mRespostesView = (TextView) view.findViewById(R.id.respostes);
                mApuntatsView = (TextView) view.findViewById(R.id.apuntats);
                mApuntarseButton = (Button) view.findViewById(R.id.boto_apuntarse);
                mNoPucButton = (Button) view.findViewById(R.id.boto_no_puc);
            }

            @Override
            public String toString() {
                return super.toString() + " '" + mLlocView.getText() + "'";
            }
        }
    }

    /**
     * Connexió a la base de dades per obtenir les dades.
     */
    public class GetDataTask extends AsyncTask<Void, Void, Boolean> {
        private List<Game> games;

        GetDataTask() {
        }

        @Override
        protected Boolean doInBackground(Void... params) {
            GlobalVars globals = ((GlobalVars)getApplication());
            Database db = new Database();
            games = db.gameList(globals, GRUP);
            return (games != null);
        }

        @Override
        protected void onPostExecute(final Boolean success) {
            mGetDataTask = null;
            if(games!=null) {
                View recyclerView = findViewById(R.id.game_list);
                assert recyclerView != null;
                setupRecyclerView((RecyclerView) recyclerView, games);
            }
        }

        @Override
        protected void onCancelled() {
            mGetDataTask = null;
        }
    }

    /**
     * Connexió a la base de dades per actualitzar camps "apuntar-se".
     */
    public class UpdateDataTask extends AsyncTask<Integer, Void, Integer> {
        UpdateDataTask() {
        }

        @Override
        protected Integer doInBackground(Integer... idPartitArray) {
            GlobalVars globals = ((GlobalVars)getApplication());
            Database db = new Database();
            int ret = db.updatePlayerGame(globals, idPartitArray[0] , OPERATION_APUNTARSE);
            return (new Integer(ret));
        }

        @Override
        protected void onPostExecute(final Integer result) {
            mUpdateDataTask = null;
            int res = result;
            if(res>0) {
                mGetDataTask = new GetDataTask();
                mGetDataTask.execute((Void) null);
            }
        }

        @Override
        protected void onCancelled() {
            mUpdateDataTask = null;
        }
    }

    /**
     * Connexió a la base de dades per actualitzar camps "no puc".
     */
    public class NoPucTask extends AsyncTask<Integer, Void, Integer> {
        NoPucTask() {
        }

        @Override
        protected Integer doInBackground(Integer... idPartitArray) {
            GlobalVars globals = ((GlobalVars)getApplication());
            Database db = new Database();
            int ret = db.updatePlayerGame(globals, idPartitArray[0] , OPERATION_NO_PUC);
            return (new Integer(ret));
        }

        @Override
        protected void onPostExecute(final Integer result) {
            mNoPucTask = null;
            int res = result;
            if(res>0) {
                mGetDataTask = new GetDataTask();
                mGetDataTask.execute((Void) null);
            }
        }

        @Override
        protected void onCancelled() {
            mNoPucTask = null;
        }
    }
}
