package com.abstractsense.paddlefriendly.presentation;

import android.content.Intent;
import android.graphics.Color;
import android.os.AsyncTask;
import android.os.Bundle;
//import android.support.design.widget.FloatingActionButton;
//import android.support.design.widget.Snackbar;
import android.support.v4.app.NavUtils;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Gravity;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;

import com.abstractsense.paddlefriendly.R;
import com.abstractsense.paddlefriendly.business.Game;
import com.abstractsense.paddlefriendly.business.Payment;
import com.abstractsense.paddlefriendly.business.Player;
import com.abstractsense.paddlefriendly.data.Database;
import com.abstractsense.paddlefriendly.data.GlobalVars;

import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import static com.abstractsense.paddlefriendly.data.Constants.FLOAT_FORMAT;
import static com.abstractsense.paddlefriendly.data.Constants.GRUP;

public class PaymentsActivity extends AppCompatActivity {

    // Assync tasks for data interchange.
    private PaymentsActivity.GetDataTask mGetDataTask = null;
    private PaymentsActivity.SaveDataTask mSaveDataTask = null;

    // Array of players.
    private String[] arrayPlayers = null;
    // Player selected.
    private int posicio = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_payments);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        /*
        FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fab);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Snackbar.make(view, "Replace with your own action", Snackbar.LENGTH_LONG)
                        .setAction("Action", null).show();
            }
        });
        */

        Spinner spinner = (Spinner) findViewById(R.id.jugador_spinner);
        spinner.setOnItemSelectedListener(new playerListener());

        Button add = (Button) findViewById(R.id.add);
        add.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                GlobalVars globals = (GlobalVars)getApplication();
                Game game = globals.getGame();
                String strQuantitat = ((TextView) findViewById(R.id.quantitat)).getText().toString();
                float quantitat = 0;
                if(!strQuantitat.equals("")) {
                    quantitat = Float.parseFloat(strQuantitat);
                }
                if(posicio > 0 && quantitat > 0) {
                    Payment payment = new Payment(0,game.getIdPartit(), "", arrayPlayers[posicio], quantitat ,"" );

                    // Afegir pagament.
                    mSaveDataTask = new PaymentsActivity.SaveDataTask();
                    mSaveDataTask.execute(payment);

                    // Reiniciar aquesta activitat per refrescar.
                    finish();
                    startActivity(getIntent());
                }
            }
        });

        mGetDataTask = new PaymentsActivity.GetDataTask();
        mGetDataTask.execute((Void) null);

        // Només el propietari pot afegeir pagaments.
        GlobalVars globals = (GlobalVars)getApplication();
        if(!globals.getGame().getPropietari().equals(globals.getPlayer().getUsua())) {
            findViewById(R.id.jugador_spinner).setEnabled(false);
            findViewById(R.id.quantitat).setEnabled(false);
            findViewById(R.id.add).setEnabled(false);
        }
    }

    private void updateTableToView(List<Payment> payments) {

        TableLayout tl = (TableLayout) findViewById(R.id.table);
        for (Iterator<Payment> i = payments.iterator(); i.hasNext(); ) {
            Payment payment = i.next();
            addTableRow(tl, payment);
        }
    }

    private void addTableRow(TableLayout tl, Payment payment) {

        TableRow tr=new TableRow(PaymentsActivity.this);
        TableLayout.LayoutParams tlp = new TableLayout.LayoutParams(TableRow.LayoutParams.WRAP_CONTENT, TableRow.LayoutParams.WRAP_CONTENT);
        tlp.setMargins(0, 15, 0, 15);
        tr.setLayoutParams(tlp);

        TextView tv = new TextView(PaymentsActivity.this);
        tv.setText(payment.getNomJugador());
        tv.setTextColor(Color.BLACK);
        tv.setTextSize(20);
        tv.setPadding(30, 0, 30, 0);
        tr.addView(tv);

        tv = new TextView(PaymentsActivity.this);
        tv.setText(String.format(FLOAT_FORMAT, payment.getQuantitat()));
        tv.setTextColor(Color.BLACK);
        tv.setTextSize(20);
        tv.setGravity(Gravity.RIGHT);
        tv.setPadding(30, 0, 30, 0);
        tr.addView(tv);

        tl.addView(tr);
    }

    private void updatePlayerList(List<Player> players) {

        String[] arrayNames = new String[players.size()+1];
        arrayNames[0] = new String("");
        String[] arrayUsers = new String[players.size()+1];
        int n = 1;

        for (Iterator<Player> i = players.iterator(); i.hasNext();n++ ) {
            Player player = i.next();
            arrayNames[n] = new String(player.getNomCognoms());
            arrayUsers[n] = new String(player.getUsua());
        }
        Spinner s = (Spinner) findViewById(R.id.jugador_spinner);
        ArrayAdapter<String> adapter = new ArrayAdapter<String>(this,
                android.R.layout.simple_spinner_item, arrayNames);
        s.setAdapter(adapter);
        arrayPlayers = arrayUsers;
    }

    /**
     * Listener for the spinner object.
     */
    public class playerListener implements AdapterView.OnItemSelectedListener {

        public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) {
            //String selected = parent.getItemAtPosition(pos).toString();
            posicio = pos;
        }

        public void onNothingSelected(AdapterView parent) {
            // Do nothing.
        }
    }

    /**
     * Obtenció de les dades.
     */
    public class GetDataTask extends AsyncTask<Void, Void, Boolean> {

        private List<Payment> payments;
        private List<Player> players;

        GetDataTask() {
        }

        @Override
        protected Boolean doInBackground(Void... params) {
            GlobalVars globals = (GlobalVars)getApplication();
            Game game = globals.getGame();
            Database db = new Database();
            payments = db.payments(globals, game);
            players = db.playerList(globals, GRUP);
            return (payments != null && players != null);
        }

        @Override
        protected void onPostExecute(final Boolean result) {
            mGetDataTask = null;
            if(result) {
                updateTableToView(payments);
                updatePlayerList(players);
            }
        }

        @Override
        protected void onCancelled() {
            mGetDataTask = null;
        }
    }

    /**
     * Connexió a la base de dades per actualitzar camps "apuntar-se" i "no puc".
     */
    public class SaveDataTask extends AsyncTask<Payment, Void, Boolean> {

        SaveDataTask() {
        }

        @Override
        protected Boolean doInBackground(Payment... payment) {

            // Save data.
            GlobalVars globals = ((GlobalVars)getApplication());
            Database db = new Database();
            boolean ret = db.addPayment(globals, payment[0]);
            return (ret);
        }

        @Override
        protected void onPostExecute(final Boolean result) {
            mSaveDataTask = null;

            // Go back to game detall.
            if(result) {
                NavUtils.navigateUpTo(PaymentsActivity.this, new Intent(PaymentsActivity.this, GameDetailActivity.class));
            } else {
                // Error message.
            }
        }

        @Override
        protected void onCancelled() {
            mSaveDataTask = null;
        }
    }

}
